@@ -1,10 +1,13 @@
 import logging
+import time
 from collections import UserList
 from datetime import date, datetime
-from typing import List
+from random import random
+from typing import Callable, List
 
 import httplib2
 from apiclient import discovery
+from apiclient.errors import HttpError
 from oauth2client.client import OAuth2Credentials
 
 
@@ -62,6 +65,27 @@ class GoogleAnalyticsAPI_V4:
             http = httplib2.Http(timeout=360)
             self.oauth_creds.refresh(http)
 
+    def request_processing(self, call_request: Callable):
+        """ Performs handling the 500 errors """
+        tries = 0
+        while tries < 5:
+            tries += 1
+            try:
+                return call_request(num_retries=1)
+            except HttpError as err:
+                # Да, я тоже ахуел, но это кусок из офф.доки:
+                #   https://developers.google.com/analytics/devguides/reporting/core/v3/errors?hl=ru
+                sleep_time = (3 ** tries) + random() * 7
+                logging.warning(f'Troubles in the hole.\n'
+                                f'Error: {err}, reason: {err.resp.reason}, text: {err.content}.\n'
+                                f'Variables:\nn: {tries}; sleep: {sleep_time}')
+                if int(err.resp.status_code) >= 500:
+                    time.sleep(sleep_time)
+                else:
+                    raise err
+
+        raise Exception("Couldn't execute the request")
+
     def get_report(self,
                    profile_id: str,
                    metrics: List[str],
@@ -99,7 +123,9 @@ class GoogleAnalyticsAPI_V4:
             ]
         }
 
-        first_response = self.client.reports().batchGet(body=params).execute()['reports'][0]
+        first_response = self.request_processing(
+            self.client.reports().batchGet(body=params).execute
+        )['reports'][0]
         total_results = int(first_response['data']['totals'][0]['values'][0])
 
         yield_handler = lambda x: [
@@ -114,7 +140,9 @@ class GoogleAnalyticsAPI_V4:
         if total_results > max_results:
             params['reportRequests'][0]['pageToken'] = first_response['nextPageToken']
             while batch_limit != 0:
-                response = self.client.reports().batchGet(body=params).execute()['reports'][0]
+                response = self.request_processing(
+                    self.client.reports().batchGet(body=params).execute
+                )['reports'][0]
 
                 yield from map(yield_handler, rows)
                 batch_limit -= 1
@@ -241,12 +269,13 @@ class GoogleAnalyticsAPI_V3:
         if not is_accessed_sampling and first_response['containsSampledData']:
             raise Exception(f'Contains sampled data. Request: {params}')
 
-        if batch_limit == 1:
-            rows = first_response.get('rows', [])
-            logging.info(f'Received {total_results} records')
-            yield from rows
+        rows = first_response.get('rows', [])
+        yield from rows
 
-        if total_results > first_response['itemsPerPage']:
+        if batch_limit == 1:
+            logging.info(f'Received {total_results} records')
+
+        elif total_results > first_response['itemsPerPage']:
             params['start_index'] = max_results - 1
             yield from rows
             while True:
